# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import re
import torch
import numpy as np
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.models.segformer import (
    SegformerForSegmentation as _SegformerForSegmentation,
)
from unitorch.models.segformer import SegformerProcessor
from unitorch.utils import pop_value, nested_dict_value
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
)
from unitorch.cli.models.segformer import pretrained_segformer_infos


class SegformerForSegmentationPipeline(_SegformerForSegmentation):
    def __init__(
        self,
        config_path: str,
        vision_config_path: str,
        weight_path: Optional[Union[str, List[str]]] = None,
        state_dict: Optional[Dict[str, Any]] = None,
        enable_cpu_offload: Optional[bool] = True,
        device: Optional[Union[str, int]] = "cpu",
    ):
        super().__init__(
            config_path=config_path,
        )
        self.processor = SegformerProcessor(
            vision_config_path=vision_config_path,
        )
        self._device = "cpu" if device == "cpu" else int(device)

        self.from_pretrained(weight_path, state_dict=state_dict)
        self._enable_cpu_offload = enable_cpu_offload
        if not self._enable_cpu_offload and self._device != "cpu":
            self.to(device=self._device)
        self.eval()

    @classmethod
    @add_default_section_for_init("core/pipeline/segformer")
    def from_core_configure(
        cls,
        config,
        pretrained_name: Optional[str] = None,
        config_path: Optional[str] = None,
        vision_config_path: Optional[str] = None,
        pretrained_weight_path: Optional[str] = None,
        device: Optional[str] = None,
        **kwargs,
    ):
        config.set_default_section("core/pipeline/segformer")
        pretrained_name = pretrained_name or config.getoption(
            "pretrained_name", "segformer-swin-tiny-ade-semantic"
        )

        config_path = config_path or config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_segformer_infos, pretrained_name, "config"),
        )
        config_path = cached_path(config_path)

        vision_config_path = vision_config_path or config.getoption(
            "vision_config_path", None
        )
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(
                pretrained_segformer_infos, pretrained_name, "vision_config"
            ),
        )
        vision_config_path = cached_path(vision_config_path)

        enable_cpu_offload = config.getoption("enable_cpu_offload", True)
        device = config.getoption("device", "cpu") if device is None else device
        pretrained_weight_path = pretrained_weight_path or config.getoption(
            "pretrained_weight_path", None
        )
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_segformer_infos, pretrained_name, "weight"),
            check_none=False,
        )

        inst = cls(
            config_path,
            vision_config_path,
            weight_path=weight_path,
            enable_cpu_offload=enable_cpu_offload,
            device=device,
        )

        return inst

    @torch.no_grad()
    @add_default_section_for_function("core/pipeline/segformer")
    def __call__(
        self,
        image: Union[Image.Image, str],
    ):
        if self._enable_cpu_offload:
            self.to(self._device)
        if isinstance(image, str):
            image = Image.open(image).convert("RGB")
        width, height = image.size
        inputs = self.processor.classification(image)
        outputs = self.forward(inputs.pixel_values.unsqueeze(0).to(self._device))
        batch = outputs.logits.shape[0]
        num_classes = outputs.logits.shape[-1]
        masks = torch.softmax(outputs.logits, dim=1)
        # set the logit of not the highest class to 0
        masks = masks * (masks == masks.max(dim=1, keepdim=True).values).float()
        classes = (
            torch.arange(num_classes, device=masks.device)
            .unsqueeze(0)
            .expand(batch, -1)
        )
        # resize masks to the original image size with nn.functional.interpolate
        masks = torch.nn.functional.interpolate(
            masks,
            size=(height, width),
            mode="bilinear",
            align_corners=False,
        )
        masks = masks.cpu().numpy()[0]
        classes = classes.cpu().numpy()[0]
        id2label = self.segformer.config.id2label
        labels = [id2label.get(int(cls), None) for cls in classes]
        if self._enable_cpu_offload:
            self.to("cpu")
            torch.cuda.empty_cache()
        return [
            (mask, label) for mask, label in zip(masks, labels) if label is not None
        ]



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import datasets

datasets.logging.set_verbosity(datasets.logging.ERROR)
from itertools import islice
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
from datasets.iterable_dataset import (
    _BaseExamplesIterable,
    SkipExamplesIterable,
    ExamplesIterable,
)
from unitorch.utils.decorators import replace


@replace(datasets.iterable_dataset.ExamplesIterable)
class ExamplesIterableV2(datasets.iterable_dataset.ExamplesIterable):
    def __init__(
        self,
        generate_examples_fn: Callable,
        kwargs: dict,
        ex_dataset_builder: Optional[datasets.GeneratorBasedBuilder] = None,
    ):
        super().__init__(generate_examples_fn, kwargs)
        self.ex_dataset_builder = ex_dataset_builder
        self.support_skip_step = hasattr(self.ex_dataset_builder, "skip")

    def __iter__(self):
        for key, example in self.generate_examples_fn(**self.kwargs):
            yield key, example

    def skip(self, n):
        self.ex_dataset_builder.skip(n)


@replace(datasets.iterable_dataset.SkipExamplesIterable)
class SkipExamplesIterableV2(datasets.iterable_dataset.SkipExamplesIterable):
    def __init__(
        self,
        ex_iterable: _BaseExamplesIterable,
        n: int,
        block_sources_order_when_shuffling: bool = True,
        split_when_sharding: bool = True,
    ):
        self.ex_iterable = ex_iterable
        self.n = n
        self.block_sources_order_when_shuffling = block_sources_order_when_shuffling
        self.split_when_sharding = split_when_sharding

    def __iter__(self):
        if (
            isinstance(self.ex_iterable, ExamplesIterable)
            and self.ex_iterable.support_skip_step
        ):
            self.ex_iterable.skip(self.n)
            self.n = 0
            ex_iterator = iter(self.ex_iterable)
        else:
            ex_iterator = iter(self.ex_iterable)
            for _ in islice(ex_iterator, self.n):
                pass
        yield from ex_iterator

    def shuffle_data_sources(self, seed: Optional[int]) -> "SkipExamplesIterable":
        """Doesn't shuffle the wrapped examples iterable since it would skip exampels from other shards instead."""
        return self



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_bart_infos = {
    "bart-base": {
        "config": hf_endpoint_url("/facebook/bart-base/resolve/main/config.json"),
        "vocab": hf_endpoint_url("/facebook/bart-base/resolve/main/vocab.json"),
        "merge": hf_endpoint_url("/facebook/bart-base/resolve/main/merges.txt"),
        "weight": hf_endpoint_url("/facebook/bart-base/resolve/main/pytorch_model.bin"),
    },
    "bart-large": {
        "config": hf_endpoint_url("/facebook/bart-large/resolve/main/config.json"),
        "vocab": hf_endpoint_url("/facebook/bart-large/resolve/main/vocab.json"),
        "merge": hf_endpoint_url("/facebook/bart-large/resolve/main/merges.txt"),
        "weight": hf_endpoint_url(
            "/facebook/bart-large/resolve/main/pytorch_model.bin"
        ),
    },
}

import unitorch.cli.models.bart.modeling
import unitorch.cli.models.bart.processing
from unitorch.cli.models.bart.modeling import BartForGeneration
from unitorch.cli.models.bart.processing import BartProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import torch.nn as nn
import pandas as pd
from dataclasses import dataclass
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.models import GenericOnnxModel
from unitorch.cli import (
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterMixin, WriterOutputs
from unitorch.cli.models import TensorsOutputs, TensorsTargets, ACT2FN


@dataclass
class PandasOutputs(WriterMixin):
    outputs: pd.DataFrame


def onnx_model_decorator(cls):
    class OnnxModel(GenericOnnxModel):
        def __init__(self, *args, **kwargs):
            super().__init__()

    return OnnxModel



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.utils import is_megatron_available

import unitorch.cli.datasets.hf

if is_megatron_available():
    import unitorch.cli.datasets.megatron



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import re
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.bloom import BloomProcessor as _BloomProcessor
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli import WriterOutputs
from unitorch.cli.models import (
    TensorsInputs,
    GenerationOutputs,
    GenerationTargets,
)
from unitorch.cli.models.bloom import pretrained_bloom_infos


class BloomProcessor(_BloomProcessor):
    """Processor for Bloom language models."""

    def __init__(
        self,
        tokenizer_file: str,
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 128,
    ):
        """
        Initialize the BloomProcessor.

        Args:
            tokenizer_file (str): The path to the tokenizer file.
            max_seq_length (int, optional): The maximum sequence length. Defaults to 128.
            max_gen_seq_length (int, optional): The maximum generation sequence length. Defaults to 128.
        """
        super().__init__(
            tokenizer_file=tokenizer_file,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )

    @classmethod
    @add_default_section_for_init("core/process/bloom")
    def from_core_configure(cls, config, **kwargs):
        """
        Create an instance of BloomProcessor from the core configuration.

        Args:
            config (Config): The core configuration object.

        Returns:
            BloomProcessor: An instance of BloomProcessor initialized with the provided configuration.
        """
        config.set_default_section("core/process/bloom")
        pretrained_name = config.getoption("pretrained_name", "bloom-560m")
        tokenizer_file = config.getoption("tokenizer_file", None)
        tokenizer_file = pop_value(
            tokenizer_file,
            nested_dict_value(pretrained_bloom_infos, pretrained_name, "tokenizer"),
        )
        tokenizer_file = cached_path(tokenizer_file)

        return {
            "tokenizer_file": tokenizer_file,
        }

    @register_process("core/process/bloom/classification")
    def _classification(
        self,
        text: str,
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        """
        Perform classification using the Bloom model.

        Args:
            text (str): The input text.
            text_pair (str, optional): The second input text for pair-based classification. Defaults to None.
            max_seq_length (int, optional): The maximum sequence length. Defaults to None.

        Returns:
            TensorsInputs: The input tensors for classification.
        """
        outputs = super().classification(
            text=text,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
        )

    @register_process("core/process/bloom/generation/inputs")
    def _generation_inputs(
        self,
        text: str,
        max_seq_length: Optional[int] = None,
    ):
        """
        Preprocess the input text for generation tasks.

        Args:
            text (str): The input text.
            max_seq_length (int, optional): The maximum sequence length. Defaults to None.

        Returns:
            TensorsInputs: The processed input tensors.
        """
        outputs = super().generation_inputs(
            text=text,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(input_ids=outputs.input_ids)

    @register_process("core/process/bloom/generation/labels")
    def _generation_labels(
        self,
        text: str,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Preprocess the target text for generation tasks.

        Args:
            text (str): The target text.
            max_gen_seq_length (int, optional): The maximum generation sequence length. Defaults to None.

        Returns:
            GenerationTargets: The processed generation targets.
        """
        outputs = super().generation_labels(
            text=text,
            max_gen_seq_length=max_gen_seq_length,
        )
        return GenerationTargets(
            refs=outputs.input_ids,
            masks=outputs.attention_mask,
        )

    @register_process("core/process/bloom/generation")
    def _generation(
        self,
        text: str,
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Preprocess the input and target texts for generation tasks.

        Args:
            text (str): The input text.
            text_pair (str, optional): The paired input text. Defaults to None.
            max_seq_length (int, optional): The maximum sequence length. Defaults to None.
            max_gen_seq_length (int, optional): The maximum generation sequence length. Defaults to None.

        Returns:
            Tuple[TensorsInputs, GenerationTargets]: The processed input tensors and generation targets.
        """
        outputs = super().generation(
            text=text,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
        ), GenerationTargets(
            refs=outputs.input_ids_label,
            masks=outputs.attention_mask_label,
        )

    @register_process("core/process/bloom/instruction/generation/inputs")
    def _instruction_generation_inputs(
        self,
        instruction: str,
        input: str,
        max_seq_length: Optional[int] = None,
    ):
        """
        Preprocess the input text for generation tasks.

        Args:
            text (str): The input text.
            max_seq_length (int, optional): The maximum sequence length. Defaults to None.

        Returns:
            TensorsInputs: The processed input tensors.
        """
        outputs = super().instruction_generation_inputs(
            instruction=instruction,
            input=input,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(input_ids=outputs.input_ids)

    @register_process("core/process/bloom/instruction/generation")
    def _instruction_generation(
        self,
        instruction: str,
        input: str,
        output: Optional[str] = None,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Preprocess the input and target texts for generation tasks.

        Args:
            input (str): The input text.
            output (str, optional): The paired input text. Defaults to None.
            max_seq_length (int, optional): The maximum sequence length. Defaults to None.
            max_gen_seq_length (int, optional): The maximum generation sequence length. Defaults to None.

        Returns:
            Tuple[TensorsInputs, GenerationTargets]: The processed input tensors and generation targets.
        """
        outputs = super().instruction_generation(
            instruction=instruction,
            input=input,
            output=output,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
        ), GenerationTargets(
            refs=outputs.input_ids_label,
            masks=outputs.attention_mask_label,
        )

    @register_process("core/postprocess/bloom/detokenize")
    def _detokenize(
        self,
        outputs: GenerationOutputs,
    ):
        """
        Detokenize the generated sequences.

        Args:
            outputs (GenerationOutputs): The generation outputs.

        Returns:
            WriterOutputs: The detokenized writer outputs.
        """
        results = outputs.to_pandas()
        assert results.shape[0] == 0 or results.shape[0] == outputs.sequences.shape[0]

        decoded = super().detokenize(sequences=outputs.sequences)
        cleanup_string = lambda text: re.sub(r"\n", " ", text)
        if isinstance(decoded[0], list):
            decoded = [list(map(cleanup_string, sequence)) for sequence in decoded]
        elif isinstance(decoded[0], str):
            decoded = list(map(cleanup_string, decoded))
        else:
            raise ValueError(
                f"Unsupported type for Bloom detokenize: {type(decoded[0])}"
            )
        results["decoded"] = decoded
        return WriterOutputs(results)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_mt5_infos = {
    "mt5-base": {
        "config": hf_endpoint_url("/google/mt5-base/resolve/main/config.json"),
        "vocab": hf_endpoint_url("/google/mt5-base/resolve/main/spiece.model"),
        "weight": hf_endpoint_url("/google/mt5-base/resolve/main/pytorch_model.bin"),
    },
    "mt5-small": {
        "config": hf_endpoint_url("/google/mt5-small/resolve/main/config.json"),
        "vocab": hf_endpoint_url("/google/mt5-small/resolve/main/spiece.model"),
        "weight": hf_endpoint_url("/google/mt5-small/resolve/main/pytorch_model.bin"),
    },
    "mt5-large": {
        "config": hf_endpoint_url("/google/mt5-large/resolve/main/config.json"),
        "vocab": hf_endpoint_url("/google/mt5-large/resolve/main/spiece.model"),
        "weight": hf_endpoint_url("/google/mt5-large/resolve/main/pytorch_model.bin"),
    },
}

import unitorch.cli.models.mt5.modeling
import unitorch.cli.models.mt5.processing
from unitorch.cli.models.mt5.modeling import MT5ForGeneration
from unitorch.cli.models.mt5.processing import MT5Processor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

from unitorch.utils import pop_value
from unitorch.cli import (
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli.models.classification_utils import ClassificationTargets


class LabelProcessor:
    """
    Processor for label-related operations.
    """

    def __init__(
        self,
        num_classes: Optional[int] = None,
        sep: Optional[str] = ",",
        max_seq_length: Optional[int] = 128,
        map_dict: Optional[Dict] = dict(),
    ):
        """
        Initializes a new instance of the LabelProcessor.

        Args:
            num_classes (Optional[int]): The number of classes. Defaults to None.
            sep (Optional[str]): The separator used for splitting text. Defaults to ",".
            max_seq_length (Optional[int]): The maximum sequence length. Defaults to 128.
            map_dict (Optional[Dict]): A dictionary for mapping labels. Defaults to an empty dictionary.
        """
        self.num_classes = num_classes
        self.sep = sep
        self.map_dict = map_dict

    @classmethod
    @add_default_section_for_init("core/process/label")
    def from_core_configure(cls, config, **kwargs):
        """
        Creates a new instance of the LabelProcessor using the configuration from the core.

        Args:
            config: The configuration object.
            **kwargs: Additional keyword arguments.

        Returns:
            An instance of the LabelProcessor.
        """
        pass

    @register_process("core/process/label")
    def _label(
        self,
        text: Union[int, float, str],
        dtype: Optional[str] = "int",
    ):
        """
        Processes a single label.

        Args:
            text (Union[int, float, str]): The label to process.
            dtype (Optional[str]): The desired data type of the output. Defaults to "int".

        Returns:
            A ClassificationTargets object containing the processed label.
        """
        if text in self.map_dict:
            text = self.map_dict[text]

        if dtype == "int":
            outputs = torch.tensor(int(text))
        else:
            outputs = torch.tensor(float(text))
        return ClassificationTargets(targets=outputs)

    @register_process("core/process/label/sequence")
    def _sequence_label(
        self,
        text: Union[List, str],
        sep: Optional[str] = None,
        dtype: Optional[str] = "int",
    ):
        """
        Processes a sequence of labels.

        Args:
            text (Union[List, str]): The sequence of labels to process.
            sep (Optional[str]): The separator used for splitting text. Defaults to None.
            dtype (Optional[str]): The desired data type of the output. Defaults to "int".

        Returns:
            A ClassificationTargets object containing the processed sequence of labels.
        """
        if isinstance(text, str):
            sep = pop_value(sep, self.sep)
            tensor = [float(self.map_dict.get(t, t)) for t in text.split(sep)]
        else:
            tensor = [float(t) for t in text]

        if dtype == "int":
            outputs = torch.tensor(tensor).int()
        else:
            outputs = torch.tensor(tensor)
        return ClassificationTargets(targets=outputs)

    @register_process("core/process/label/binary")
    def _binary_label(
        self,
        text: Union[List, str],
        sep: Optional[str] = None,
    ):
        """
        Processes binary labels.

        Args:
            text (Union[List, str]): The binary labels to process.
            sep (Optional[str]): The separator used for splitting text. Defaults to None.

        Returns:
            A ClassificationTargets object containing the processed binary labels.
        """
        outputs = torch.zeros(self.num_classes)
        if isinstance(text, str):
            sep = pop_value(sep, self.sep)
            indexes = [int(self.map_dict.get(t, t)) for t in text.split(sep)]
        else:
            indexes = [int(t) for t in text]
        outputs[indexes] = 1
        return ClassificationTargets(targets=outputs)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import time
import logging
import http.server
import zipfile
from urllib.parse import parse_qs, urlparse
from unitorch.cli import CoreConfigureParser
from unitorch.cli import register_service, GenericService


class HttpFilesServer(http.server.SimpleHTTPRequestHandler):
    html_dir = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, directory=self.html_dir, **kwargs)


@register_service("core/service/http_files")
class HttpFilesService(GenericService):
    def __init__(self, config: CoreConfigureParser):
        self.config = config
        config.set_default_section("core/service/http_files")
        self.ip = config.getoption("ip", "0.0.0.0")
        self.port = config.getoption("port", 11220)
        self.name = config.getoption("processname", "core_http_files_service")
        self.html_dir = config.getoption("html_dir", None)
        assert self.html_dir is not None, "html_dir must be provided"

    def start(self, **kwargs):
        HttpFilesServer.html_dir = self.html_dir
        self.httpd = http.server.HTTPServer((self.ip, self.port), HttpFilesServer)
        self.httpd.serve_forever()

    def stop(self, **kwargs):
        pass

    def restart(self, **kwargs):
        pass



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import json
import torch
import torch.nn.functional as F
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
import diffusers.schedulers as schedulers
from transformers import CLIPTextConfig, CLIPTextModel, CLIPTextModelWithProjection
from transformers.models.t5.configuration_t5 import T5Config
from transformers.models.t5.modeling_t5 import T5EncoderModel
from diffusers.schedulers import SchedulerMixin, FlowMatchEulerDiscreteScheduler
from diffusers.training_utils import (
    compute_loss_weighting_for_sd3,
    compute_density_for_timestep_sampling,
)
from diffusers.models import (
    SD3ControlNetModel,
    SD3MultiControlNetModel,
    SD3Transformer2DModel,
    AutoencoderKL,
)
from diffusers.pipelines import (
    StableDiffusion3Pipeline,
    StableDiffusion3Img2ImgPipeline,
    StableDiffusion3InpaintPipeline,
)
from unitorch.models import (
    GenericModel,
    GenericOutputs,
    QuantizationConfig,
    QuantizationMixin,
)
from unitorch.models.peft import PeftWeightLoaderMixin
from unitorch.models.diffusers import compute_snr


class GenericStable3Model(GenericModel, QuantizationMixin, PeftWeightLoaderMixin):
    prefix_keys_in_state_dict = {
        # vae weights
        "^encoder.*": "vae.",
        "^decoder.*": "vae.",
        "^post_quant_conv.*": "vae.",
        "^quant_conv.*": "vae.",
    }

    replace_keys_in_state_dict = {
        "\.query\.": ".to_q.",
        "\.key\.": ".to_k.",
        "\.value\.": ".to_v.",
        "\.proj_attn\.": ".to_out.0.",
    }

    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        text2_config_path: str,
        text3_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        controlnet_configs_path: Union[str, List[str]] = None,
        inpainting_controlnet_config_path: Union[str] = None,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        freeze_transformer_encoder: Optional[bool] = False,
        snr_gamma: Optional[float] = 5.0,
        seed: Optional[int] = 1123,
    ):
        super().__init__()
        self.seed = seed
        self.num_train_timesteps = num_train_timesteps
        self.num_infer_timesteps = num_infer_timesteps
        self.snr_gamma = snr_gamma

        config_dict = json.load(open(config_path))
        if image_size is not None:
            config_dict.update({"sample_size": image_size})
        if in_channels is not None:
            config_dict.update({"in_channels": in_channels})
        if out_channels is not None:
            config_dict.update({"out_channels": out_channels})
        self.transformer = SD3Transformer2DModel.from_config(config_dict)

        text_config = CLIPTextConfig.from_json_file(text_config_path)
        self.text = CLIPTextModelWithProjection(text_config)

        text_config2 = CLIPTextConfig.from_json_file(text2_config_path)
        self.text2 = CLIPTextModelWithProjection(text_config2)

        text_config3 = T5Config.from_json_file(text3_config_path)
        self.text3 = T5EncoderModel(text_config3)

        vae_config_dict = json.load(open(vae_config_path))
        self.vae = AutoencoderKL.from_config(vae_config_dict)

        if isinstance(controlnet_configs_path, str):
            controlnet_configs_path = [controlnet_configs_path]
        if isinstance(inpainting_controlnet_config_path, str):
            controlnet_configs_path += [inpainting_controlnet_config_path]

        if isinstance(controlnet_configs_path, list):
            if len(controlnet_configs_path) == 0:
                controlnet_configs_path = None
            elif len(controlnet_configs_path) == 1:
                controlnet_configs_path = controlnet_configs_path[0]

        if isinstance(controlnet_configs_path, list):
            controlnets = []
            for controlnet_config_path in controlnet_configs_path:
                controlnet_config_dict = json.load(open(controlnet_config_path))
                controlnets.append(
                    SD3ControlNetModel.from_config(controlnet_config_dict)
                )
            self.num_controlnets = len(controlnets)
            self.controlnet = SD3MultiControlNetModel(
                controlnets=controlnets,
            )
        elif isinstance(controlnet_configs_path, str):
            controlnet_config_dict = json.load(open(controlnet_configs_path))
            self.controlnet = SD3ControlNetModel.from_config(controlnet_config_dict)
            self.num_controlnets = 1
        else:
            self.controlnet = None
            self.num_controlnets = 0

        scheduler_config_dict = json.load(open(scheduler_config_path))
        scheduler_class_name = scheduler_config_dict.get(
            "_class_name", "FlowMatchEulerDiscreteScheduler"
        )
        assert hasattr(schedulers, scheduler_class_name)
        scheduler_class = getattr(schedulers, scheduler_class_name)
        assert issubclass(scheduler_class, SchedulerMixin)
        scheduler_config_dict["num_train_timesteps"] = num_train_timesteps
        self.scheduler = scheduler_class.from_config(scheduler_config_dict)

        if freeze_vae_encoder:
            for param in self.vae.parameters():
                param.requires_grad = False

        if freeze_text_encoder:
            for param in self.text.parameters():
                param.requires_grad = False
            for param in self.text2.parameters():
                param.requires_grad = False
            for param in self.text3.parameters():
                param.requires_grad = False

        if freeze_transformer_encoder:
            for param in self.transformer.parameters():
                param.requires_grad = False

        if quant_config_path is not None:
            self.quant_config = QuantizationConfig.from_json_file(quant_config_path)
            self.quantize(
                self.quant_config, ignore_modules=["lm_head", "transformer", "vae"]
            )

    def get_sigmas(self, timesteps, n_dim=4, dtype=torch.float32):
        sigmas = self.scheduler.sigmas.to(device=self.device, dtype=dtype)
        schedule_timesteps = self.scheduler.timesteps.to(self.device)
        timesteps = timesteps.to(self.device)
        step_indices = [(schedule_timesteps == t).nonzero().item() for t in timesteps]

        sigma = sigmas[step_indices].flatten()
        while len(sigma.shape) < n_dim:
            sigma = sigma.unsqueeze(-1)
        return sigma

    def get_prompt_outputs(
        self,
        input_ids: torch.Tensor,
        input2_ids: torch.Tensor,
        input3_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        negative_input2_ids: torch.Tensor,
        negative_input3_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        attention2_mask: Optional[torch.Tensor] = None,
        attention3_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        negative_attention2_mask: Optional[torch.Tensor] = None,
        negative_attention3_mask: Optional[torch.Tensor] = None,
        enable_cpu_offload: Optional[bool] = False,
        cpu_offload_device: Optional[str] = "cpu",
    ):
        if enable_cpu_offload:
            self.text = self.text.to(cpu_offload_device)
            self.text2 = self.text2.to(cpu_offload_device)
            self.text3 = self.text3.to(cpu_offload_device)
            input_ids = input_ids.to(cpu_offload_device)
            input2_ids = input2_ids.to(cpu_offload_device)
            input3_ids = input3_ids.to(cpu_offload_device)
            negative_input_ids = negative_input_ids.to(cpu_offload_device)
            negative_input2_ids = negative_input2_ids.to(cpu_offload_device)
            negative_input3_ids = negative_input3_ids.to(cpu_offload_device)

        prompt_outputs = self.text(
            input_ids,
            # attention_mask,
            output_hidden_states=True,
        )
        pooled_prompt_embeds = prompt_outputs[0]
        prompt_embeds = prompt_outputs.hidden_states[-2]
        negative_prompt_outputs = self.text(
            negative_input_ids,
            # negative_attention_mask,
            output_hidden_states=True,
        )
        negative_pooled_prompt_embeds = negative_prompt_outputs[0]
        negative_prompt_embeds = negative_prompt_outputs.hidden_states[-2]

        prompt2_outputs = self.text2(
            input2_ids,
            # attention2_mask,
            output_hidden_states=True,
        )
        pooled_prompt2_embeds = prompt2_outputs[0]
        prompt2_embeds = prompt2_outputs.hidden_states[-2]
        negative_prompt2_outputs = self.text2(
            negative_input2_ids,
            # negative_attention2_mask,
            output_hidden_states=True,
        )
        negative_pooled_prompt2_embeds = negative_prompt2_outputs[0]
        negative_prompt2_embeds = negative_prompt2_outputs.hidden_states[-2]

        prompt3_outputs = self.text3(
            input3_ids,
            # attention3_mask,
            output_hidden_states=True,
        )
        prompt3_embeds = prompt3_outputs[0]
        negative_prompt3_outputs = self.text3(
            negative_input3_ids,
            # negative_attention3_mask,
            output_hidden_states=True,
        )
        negative_prompt3_embeds = negative_prompt3_outputs[0]

        prompt_embeds = torch.concat([prompt_embeds, prompt2_embeds], dim=-1)
        negative_prompt_embeds = torch.concat(
            [negative_prompt_embeds, negative_prompt2_embeds], dim=-1
        )
        pooled_prompt_embeds = torch.concat(
            [pooled_prompt_embeds, pooled_prompt2_embeds], dim=-1
        )
        negative_pooled_prompt_embeds = torch.concat(
            [negative_pooled_prompt_embeds, negative_pooled_prompt2_embeds], dim=-1
        )

        prompt_embeds = torch.nn.functional.pad(
            prompt_embeds, (0, prompt3_embeds.shape[-1] - prompt_embeds.shape[-1])
        )
        prompt_embeds = torch.cat([prompt_embeds, prompt3_embeds], dim=-2)
        negative_prompt_embeds = torch.nn.functional.pad(
            negative_prompt_embeds,
            (0, negative_prompt3_embeds.shape[-1] - negative_prompt_embeds.shape[-1]),
        )
        negative_prompt_embeds = torch.cat(
            [negative_prompt_embeds, negative_prompt3_embeds], dim=-2
        )
        if enable_cpu_offload:
            self.text = self.text.to("cpu")
            self.text2 = self.text2.to("cpu")
            self.text3 = self.text3.to("cpu")

        return GenericOutputs(
            prompt_embeds=prompt_embeds.to("cpu")
            if enable_cpu_offload
            else prompt_embeds,
            negative_prompt_embeds=negative_prompt_embeds.to("cpu")
            if enable_cpu_offload
            else negative_prompt_embeds,
            pooled_prompt_embeds=pooled_prompt_embeds.to("cpu")
            if enable_cpu_offload
            else pooled_prompt_embeds,
            negative_pooled_prompt_embeds=negative_pooled_prompt_embeds.to("cpu")
            if enable_cpu_offload
            else negative_pooled_prompt_embeds,
        )


class Stable3ForText2ImageGeneration(GenericStable3Model):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        text2_config_path: str,
        text3_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            text2_config_path=text2_config_path,
            text3_config_path=text3_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            seed=seed,
        )

        self.pipeline = StableDiffusion3Pipeline(
            vae=self.vae,
            text_encoder=self.text,
            text_encoder_2=self.text2,
            text_encoder_3=self.text3,
            transformer=self.transformer,
            scheduler=self.scheduler,
            tokenizer=None,
            tokenizer_2=None,
            tokenizer_3=None,
        )
        self.pipeline.set_progress_bar_config(disable=True)

    def forward(
        self,
        input_ids: torch.Tensor,
        input2_ids: torch.Tensor,
        input3_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        attention2_mask: Optional[torch.Tensor] = None,
        attention3_mask: Optional[torch.Tensor] = None,
    ):
        prompt_outputs = self.text(
            input_ids,
            # attention_mask,
            output_hidden_states=True,
        )
        pooled_prompt_embeds = prompt_outputs[0]
        prompt_embeds = prompt_outputs.hidden_states[-2]

        prompt2_outputs = self.text2(
            input2_ids,
            # attention2_mask,
            output_hidden_states=True,
        )
        pooled_prompt2_embeds = prompt2_outputs[0]
        prompt2_embeds = prompt2_outputs.hidden_states[-2]

        prompt3_outputs = self.text3(
            input3_ids,
            # attention3_mask,
            output_hidden_states=True,
        )
        prompt3_embeds = prompt3_outputs[0]

        prompt_embeds = torch.concat([prompt_embeds, prompt2_embeds], dim=-1)
        pooled_prompt_embeds = torch.concat(
            [pooled_prompt_embeds, pooled_prompt2_embeds], dim=-1
        )

        prompt_embeds = torch.nn.functional.pad(
            prompt_embeds, (0, prompt3_embeds.shape[-1] - prompt_embeds.shape[-1])
        )
        prompt_embeds = torch.cat([prompt_embeds, prompt3_embeds], dim=-2)

        latents = self.vae.encode(pixel_values).latent_dist.sample()
        latents = latents * self.vae.config.scaling_factor

        noise = torch.randn(latents.shape).to(latents.device)
        batch = latents.size(0)

        u = compute_density_for_timestep_sampling(
            weighting_scheme="none",
            batch_size=batch,
            logit_mean=0.0,
            logit_std=1.0,
            mode_scale=1.29,
        )
        indices = (u * self.scheduler.config.num_train_timesteps).long()
        timesteps = self.scheduler.timesteps[indices].to(device=self.device)

        sigmas = self.get_sigmas(timesteps, n_dim=latents.ndim, dtype=latents.dtype)
        noise_latents = (1.0 - sigmas) * latents + sigmas * noise

        outputs = self.transformer(
            noise_latents,
            timestep=timesteps,
            encoder_hidden_states=prompt_embeds,
            pooled_projections=pooled_prompt_embeds,
        ).sample

        weighting = compute_loss_weighting_for_sd3(
            weighting_scheme="none", sigmas=sigmas
        )
        target = noise - latents
        loss = torch.mean(
            (weighting.float() * (outputs.float() - target.float()) ** 2).reshape(
                target.shape[0], -1
            ),
            1,
        )
        loss = loss.mean()
        return loss

    def generate(
        self,
        input_ids: torch.Tensor,
        input2_ids: torch.Tensor,
        input3_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        negative_input2_ids: torch.Tensor,
        negative_input3_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        attention2_mask: Optional[torch.Tensor] = None,
        attention3_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        negative_attention2_mask: Optional[torch.Tensor] = None,
        negative_attention3_mask: Optional[torch.Tensor] = None,
        height: Optional[int] = 1024,
        width: Optional[int] = 1024,
        guidance_scale: Optional[float] = 5.0,
    ):
        outputs = self.get_prompt_outputs(
            input_ids=input_ids,
            input2_ids=input2_ids,
            input3_ids=input3_ids,
            negative_input_ids=negative_input_ids,
            negative_input2_ids=negative_input2_ids,
            negative_input3_ids=negative_input3_ids,
            attention_mask=attention_mask,
            attention2_mask=attention2_mask,
            attention3_mask=attention3_mask,
            negative_attention_mask=negative_attention_mask,
            negative_attention2_mask=negative_attention2_mask,
            negative_attention3_mask=negative_attention3_mask,
        )

        images = self.pipeline(
            prompt_embeds=outputs.prompt_embeds,
            negative_prompt_embeds=outputs.negative_prompt_embeds,
            pooled_prompt_embeds=outputs.pooled_prompt_embeds,
            negative_pooled_prompt_embeds=outputs.negative_pooled_prompt_embeds,
            generator=torch.Generator(device=self.pipeline.device).manual_seed(
                self.seed
            ),
            num_inference_steps=self.num_infer_timesteps,
            height=height,
            width=width,
            guidance_scale=guidance_scale,
            output_type="np.array",
        ).images

        return GenericOutputs(images=torch.from_numpy(images))


class Stable3ForImage2ImageGeneration(GenericStable3Model):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        text2_config_path: str,
        text3_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            text2_config_path=text2_config_path,
            text3_config_path=text3_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            seed=seed,
        )

        self.pipeline = StableDiffusion3Img2ImgPipeline(
            vae=self.vae,
            text_encoder=self.text,
            text_encoder_2=self.text2,
            text_encoder_3=self.text3,
            transformer=self.transformer,
            scheduler=self.scheduler,
            tokenizer=None,
            tokenizer_2=None,
            tokenizer_3=None,
        )
        self.pipeline.set_progress_bar_config(disable=True)

    def forward(
        self,
    ):
        raise NotImplementedError

    def generate(
        self,
        pixel_values: torch.Tensor,
        input_ids: torch.Tensor,
        input2_ids: torch.Tensor,
        input3_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        negative_input2_ids: torch.Tensor,
        negative_input3_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        attention2_mask: Optional[torch.Tensor] = None,
        attention3_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        negative_attention2_mask: Optional[torch.Tensor] = None,
        negative_attention3_mask: Optional[torch.Tensor] = None,
        strength: Optional[float] = 1.0,
        guidance_scale: Optional[float] = 7.5,
    ):
        outputs = self.get_prompt_outputs(
            input_ids=input_ids,
            input2_ids=input2_ids,
            input3_ids=input3_ids,
            negative_input_ids=negative_input_ids,
            negative_input2_ids=negative_input2_ids,
            negative_input3_ids=negative_input3_ids,
            attention_mask=attention_mask,
            attention2_mask=attention2_mask,
            attention3_mask=attention3_mask,
            negative_attention_mask=negative_attention_mask,
            negative_attention2_mask=negative_attention2_mask,
            negative_attention3_mask=negative_attention3_mask,
        )

        images = self.pipeline(
            image=pixel_values,
            prompt_embeds=outputs.prompt_embeds,
            negative_prompt_embeds=outputs.negative_prompt_embeds,
            pooled_prompt_embeds=outputs.pooled_prompt_embeds,
            negative_pooled_prompt_embeds=outputs.negative_pooled_prompt_embeds,
            generator=torch.Generator(device=self.pipeline.device).manual_seed(
                self.seed
            ),
            num_inference_steps=self.num_infer_timesteps,
            strength=strength,
            guidance_scale=guidance_scale,
            output_type="np.array",
        ).images

        return GenericOutputs(images=torch.from_numpy(images))


class Stable3ForImageInpainting(GenericStable3Model):
    def __init__(
        self,
        config_path: str,
        text_config_path: str,
        text2_config_path: str,
        text3_config_path: str,
        vae_config_path: str,
        scheduler_config_path: str,
        quant_config_path: Optional[str] = None,
        image_size: Optional[int] = None,
        in_channels: Optional[int] = None,
        out_channels: Optional[int] = None,
        num_train_timesteps: Optional[int] = 1000,
        num_infer_timesteps: Optional[int] = 50,
        freeze_vae_encoder: Optional[bool] = True,
        freeze_text_encoder: Optional[bool] = True,
        snr_gamma: Optional[float] = 5.0,
        seed: Optional[int] = 1123,
    ):
        super().__init__(
            config_path=config_path,
            text_config_path=text_config_path,
            text2_config_path=text2_config_path,
            text3_config_path=text3_config_path,
            vae_config_path=vae_config_path,
            scheduler_config_path=scheduler_config_path,
            quant_config_path=quant_config_path,
            image_size=image_size,
            in_channels=in_channels,
            out_channels=out_channels,
            num_train_timesteps=num_train_timesteps,
            num_infer_timesteps=num_infer_timesteps,
            freeze_vae_encoder=freeze_vae_encoder,
            freeze_text_encoder=freeze_text_encoder,
            snr_gamma=snr_gamma,
            seed=seed,
        )

        self.pipeline = StableDiffusion3InpaintPipeline(
            vae=self.vae,
            text_encoder=self.text,
            text_encoder_2=self.text2,
            text_encoder_3=self.text3,
            transformer=self.transformer,
            scheduler=self.scheduler,
            tokenizer=None,
            tokenizer_2=None,
            tokenizer_3=None,
        )
        self.pipeline.set_progress_bar_config(disable=True)
        self.num_channels_transformer = self.transformer.config.in_channels

    def forward(
        self,
        input_ids: torch.Tensor,
        input2_ids: torch.Tensor,
        input3_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        pixel_masks: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        attention2_mask: Optional[torch.Tensor] = None,
        attention3_mask: Optional[torch.Tensor] = None,
    ):
        prompt_outputs = self.text(
            input_ids,
            # attention_mask,
            output_hidden_states=True,
        )
        pooled_prompt_embeds = prompt_outputs[0]
        prompt_embeds = prompt_outputs.hidden_states[-2]

        prompt2_outputs = self.text2(
            input2_ids,
            # attention2_mask,
            output_hidden_states=True,
        )
        pooled_prompt2_embeds = prompt2_outputs[0]
        prompt2_embeds = prompt2_outputs.hidden_states[-2]

        prompt3_outputs = self.text3(
            input3_ids,
            # attention3_mask,
            output_hidden_states=True,
        )
        prompt3_embeds = prompt3_outputs[0]

        prompt_embeds = torch.concat([prompt_embeds, prompt2_embeds], dim=-1)
        pooled_prompt_embeds = torch.concat(
            [pooled_prompt_embeds, pooled_prompt2_embeds], dim=-1
        )

        prompt_embeds = torch.nn.functional.pad(
            prompt_embeds, (0, prompt3_embeds.shape[-1] - prompt_embeds.shape[-1])
        )
        prompt_embeds = torch.cat([prompt_embeds, prompt3_embeds], dim=-2)

        latents = self.vae.encode(pixel_values).latent_dist.sample()
        latents = latents * self.vae.config.scaling_factor

        noise = torch.randn(latents.shape).to(latents.device)
        batch = latents.size(0)

        u = compute_density_for_timestep_sampling(
            weighting_scheme="none",
            batch_size=batch,
            logit_mean=0.0,
            logit_std=1.0,
            mode_scale=1.29,
        )
        indices = (u * self.scheduler.config.num_train_timesteps).long()
        timesteps = self.scheduler.timesteps[indices].to(device=self.device)

        sigmas = self.get_sigmas(timesteps, n_dim=latents.ndim, dtype=latents.dtype)
        noise_latents = (1.0 - sigmas) * latents + sigmas * noise

        if self.num_channels_transformer == 33:
            masked_pixel_values = pixel_values.clone()
            masked_pixel_masks = pixel_masks.clone()
            masked_pixel_masks = masked_pixel_masks.expand_as(masked_pixel_values)
            masked_pixel_values[masked_pixel_masks > 0.5] = 0.0
            masked_latents = self.vae.encode(masked_pixel_values).latent_dist.sample()
            masked_latents = masked_latents * self.vae.config.scaling_factor

            pixel_masks = torch.nn.functional.interpolate(
                pixel_masks, size=latents.shape[-2:], mode="nearest"
            )

            latent_model_input = torch.cat(
                [noise_latents, pixel_masks, masked_latents], dim=1
            )
        else:
            latent_model_input = noise_latents

        outputs = self.transformer(
            latent_model_input,
            timestep=timesteps,
            encoder_hidden_states=prompt_embeds,
            pooled_projections=pooled_prompt_embeds,
        ).sample

        weighting = compute_loss_weighting_for_sd3(
            weighting_scheme="none", sigmas=sigmas
        )
        target = noise - latents
        loss = torch.mean(
            (weighting.float() * (outputs.float() - target.float()) ** 2).reshape(
                target.shape[0], -1
            ),
            1,
        )
        loss = loss.mean()
        return loss

    def generate(
        self,
        pixel_values: torch.Tensor,
        pixel_masks: torch.Tensor,
        input_ids: torch.Tensor,
        input2_ids: torch.Tensor,
        input3_ids: torch.Tensor,
        negative_input_ids: torch.Tensor,
        negative_input2_ids: torch.Tensor,
        negative_input3_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        attention2_mask: Optional[torch.Tensor] = None,
        attention3_mask: Optional[torch.Tensor] = None,
        negative_attention_mask: Optional[torch.Tensor] = None,
        negative_attention2_mask: Optional[torch.Tensor] = None,
        negative_attention3_mask: Optional[torch.Tensor] = None,
        strength: Optional[float] = 1.0,
        guidance_scale: Optional[float] = 7.5,
    ):
        outputs = self.get_prompt_outputs(
            input_ids=input_ids,
            input2_ids=input2_ids,
            input3_ids=input3_ids,
            negative_input_ids=negative_input_ids,
            negative_input2_ids=negative_input2_ids,
            negative_input3_ids=negative_input3_ids,
            attention_mask=attention_mask,
            attention2_mask=attention2_mask,
            attention3_mask=attention3_mask,
            negative_attention_mask=negative_attention_mask,
            negative_attention2_mask=negative_attention2_mask,
            negative_attention3_mask=negative_attention3_mask,
        )

        images = self.pipeline(
            image=pixel_values,
            mask_image=pixel_masks,
            prompt_embeds=outputs.prompt_embeds,
            negative_prompt_embeds=outputs.negative_prompt_embeds,
            pooled_prompt_embeds=outputs.pooled_prompt_embeds,
            negative_pooled_prompt_embeds=outputs.negative_pooled_prompt_embeds,
            generator=torch.Generator(device=self.pipeline.device).manual_seed(
                self.seed
            ),
            num_inference_steps=self.num_infer_timesteps,
            width=pixel_values.size(-1),
            height=pixel_values.size(-2),
            strength=strength,
            guidance_scale=guidance_scale,
            output_type="np.array",
        ).images

        return GenericOutputs(images=torch.from_numpy(images))



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import io
import cv2
import imageio
import requests
import time
import base64
import json
import logging
import torch
import torch.nn as nn
import numpy as np
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from random import random
from PIL import Image, ImageOps, ImageFile, ImageFilter
from unitorch.utils import is_opencv_available
from unitorch.cli import (
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)

ImageFile.LOAD_TRUNCATED_IMAGES = True


class VideoProcessor:
    """
    Processor for image-related operations.
    """

    def __init__(
        self,
        video_type: Optional[str] = None,
        video_size: tuple = (256, 256),
        http_url: Optional[str] = None,
    ):
        """
        Initializes a new instance of the ImageProcessor.

        Args:
            image_type (Optional[str]): The type of the image. Defaults to None.
            image_size (tuple): The size of the image. Defaults to (256, 256).
            http_url (Optional[str]): The URL for fetching images. Defaults to None.
        """
        self.video_type = video_type
        self.video_size = video_size
        self.http_url = http_url

    @classmethod
    @add_default_section_for_init("core/process/video")
    def from_core_configure(cls, config, **kwargs):
        """
        Creates a new instance of the ImageProcessor using the configuration from the core.

        Args:
            config: The configuration object.
            **kwargs: Additional keyword arguments.

        Returns:
            An instance of the ImageProcessor.
        """
        pass

    def _request_url(self, url):
        """
        Sends an HTTP request to the specified URL and returns the response.

        Args:
            url (str): The URL to request.

        Returns:
            The response received from the URL.
        """
        while True:
            try:
                doc = requests.get(url, timeout=600)
                return doc
            except:
                time.sleep(random() * 2)

    @register_process("core/process/video/read")
    def _read(
        self,
        video,
        video_type=None,
    ):
        """
        Reads and processes an image.

        Args:
            image: The image to read and process.
            image_type (Optional[str]): The type of the image. Defaults to None.

        Returns:
            The processed image as a PIL Image object.
        """
        video_type = video_type if video_type is not None else self.video_type
        try:
            if video_type == "base64":
                video = io.BytesIO(base64.b64decode(video))

                frames = []
                with imageio.v3.imopen(
                    video, "r", plugin="pyav", format="mp4"
                ) as reader:
                    for frame in reader.iter():
                        img = Image.fromarray(frame)
                        frames.append(img)
                return frames

            if video_type == "hex":
                video = io.BytesIO(bytes.fromhex(video))

                frames = []
                with imageio.v3.imopen(
                    video, "r", plugin="pyav", format="mp4"
                ) as reader:
                    for frame in reader.iter():
                        img = Image.fromarray(frame)
                        frames.append(img)
                return frames

            if self.http_url is None:
                video = cv2.VideoCapture(video)
                return video

            url = self.http_url.format(video)
            doc = self._request_url(url)
            if doc.status_code != 200 or doc.content == b"":
                raise ValueError(f"can't find the video {video}")

            video = io.BytesIO(doc.content)
            frames = []
            with imageio.v3.imopen(video, "r", plugin="pyav", format="mp4") as reader:
                for frame in reader.iter():
                    img = Image.fromarray(frame)
                    frames.append(img)
            return frames

        except:
            logging.debug(f"core/process/video/read use fake video for {video}")
            return []

    @register_process("core/process/video/sample")
    def _sample(
        self,
        video: Union[cv2.VideoCapture, str, List[Image.Image]],
        freq: Optional[int] = 0,
        num: Optional[int] = None,
        mode: Optional[str] = "random",
    ):
        """
        Samples frames from a video.

        Args:
            video: The video to sample frames from.
            freq (Optional[int]): The frequency of frames to sample. Defaults to 0.
            num (Optional[int]): The number of frames to sample. Defaults to None.
            mode (Optional[str]): The mode of sampling. Defaults to "random".
        Returns:
            A list of sampled frames.
        """
        if isinstance(video, str):
            video = cv2.VideoCapture(video)

        if isinstance(video, cv2.VideoCapture):
            frames = []
            while video.isOpened():
                ret, frame = video.read()
                if not ret:
                    break
                rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
                pil_img = Image.fromarray(rgb)
                frames.append(pil_img)
        else:
            frames = video

        if freq > 0:
            frames = frames[:: (freq + 1)]

        if num is not None and len(frames) > num:
            if mode == "random":
                start = int(random() * (len(frames) - num))
                end = start + num
                frames = frames[start:end]
            elif mode == "first":
                frames = frames[:num]
            elif mode == "last":
                frames = frames[-num:]
            elif mode == "middle":
                start = (len(frames) - num) // 2
                end = start + num
                frames = frames[start:end]
            else:
                raise ValueError(f"Unknown sampling mode: {mode}")

        return frames



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import json
import random
import logging
import torch
import torch.nn.functional as F
import torch.nn as nn
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

from transformers.models.dinov2 import Dinov2Config, Dinov2Model
from unitorch.models import GenericModel


class DinoV2ForImageClassification(GenericModel):
    prefix_keys_in_state_dict = {
        "^embeddings.": "dinov2.",
        "^layernorm.": "dinov2.",
        "^encoder.": "dinov2.",
    }
    """
    ViT model for image classification tasks.
    """

    def __init__(
        self,
        config_path: str,
        num_classes: Optional[int] = 1,
    ):
        """
        Initializes the ViTForImageClassification model.

        Args:
            config_path (str): Path to the configuration file.
            num_classes (Optional[int]): Number of classes. Defaults to 1.
        """
        super().__init__()
        config = Dinov2Config.from_json_file(config_path)

        self.dinov2 = Dinov2Model(config)
        self.classifier = nn.Linear(config.hidden_size * 2, num_classes)
        self.init_weights()

    def forward(
        self,
        pixel_values: torch.Tensor,
    ):
        """
        Forward pass of the ViTForImageClassification model.

        Args:
            pixel_values (torch.Tensor): Input tensor of shape [batch_size, num_channels, height, width].

        Returns:
            (torch.Tensor):Output logits of shape [batch_size, num_classes].
        """
        vision_outputs = self.dinov2(
            pixel_values=pixel_values,
        )[0]
        cls_output = vision_outputs[:, 0]
        seq_output = vision_outputs[:, 1:]
        pooled_output = torch.cat([cls_output, seq_output.mean(dim=1)], dim=-1)
        return self.classifier(pooled_output)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from torch import autocast
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.kolors import (
    KolorsMPSModel as _KolorsMPSModel,
)
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_model,
)
from unitorch.cli.models import ClassificationOutputs, LossOutputs
from unitorch.cli.models.kolors import pretrained_kolors_infos


@register_model("core/model/classification/kolors/mps")
class KolorsMPSModel(_KolorsMPSModel):
    """CLIP model for classification."""

    def __init__(self, config_path: str):
        """
        Initialize the ClipForClassification model.

        Args:
            config_path (str): The path to the model configuration file.
            projection_dim (int, optional): The dimension of the projection head. Defaults to 512.
            num_classes (int, optional): The number of output classes. Defaults to 1.
            freeze_base_model (bool, optional): Whether to freeze the base model. Defaults to True.
            gradient_checkpointing (bool, optional): Whether to use gradient checkpointing. Defaults to False.
        """
        super().__init__(
            config_path=config_path,
        )

    @classmethod
    @add_default_section_for_init("core/model/classification/kolors/mps")
    def from_core_configure(cls, config, **kwargs):
        """
        Create an instance of ClipForClassification from a core configuration.

        Args:
            config: The core configuration.
            **kwargs: Additional keyword arguments.

        Returns:
            ClipForClassification: An instance of the ClipForClassification model.
        """
        config.set_default_section("core/model/classification/kolors/mps")
        pretrained_name = config.getoption("pretrained_name", "kolors-mps-overall")
        config_path = config.getoption("config_path", None)
        config_path = pop_value(
            config_path,
            nested_dict_value(pretrained_kolors_infos, pretrained_name, "config"),
        )

        config_path = cached_path(config_path)

        inst = cls(
            config_path=config_path,
        )
        pretrained_weight_path = config.getoption("pretrained_weight_path", None)
        weight_path = pop_value(
            pretrained_weight_path,
            nested_dict_value(pretrained_kolors_infos, pretrained_name, "weight"),
            check_none=False,
        )
        if weight_path is not None:
            inst.from_pretrained(weight_path)

        return inst

    @autocast(device_type=("cuda" if torch.cuda.is_available() else "cpu"))
    def forward(
        self,
        input_ids: torch.Tensor,
        pixel_values: torch.Tensor,
        condition_input_ids: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.Tensor] = None,
        condition_attention_mask: Optional[torch.Tensor] = None,
        condition_position_ids: Optional[torch.Tensor] = None,
        labels: Optional[torch.Tensor] = None,
    ):
        """
        Perform a forward pass through the model.

        Args:
            input_ids (torch.Tensor): Input token IDs.
            pixel_values (torch.Tensor): Input pixel values.
            attention_mask (torch.Tensor, optional): Attention mask. Defaults to None.
            position_ids (torch.Tensor, optional): Position IDs. Defaults to None.

        Returns:
            ClassificationOutputs: The classification outputs.
        """
        if self.training:
            assert labels is not None
            outputs = super().forward(
                input_ids=input_ids,
                pixel_values=pixel_values,
                attention_mask=attention_mask,
                position_ids=position_ids,
                condition_input_ids=condition_input_ids,
                condition_attention_mask=condition_attention_mask,
                condition_position_ids=condition_position_ids,
                labels=labels,
            )
            return LossOutputs(outputs=outputs)

        outputs = super().forward(
            input_ids=input_ids,
            pixel_values=pixel_values,
            attention_mask=attention_mask,
            position_ids=position_ids,
            condition_input_ids=condition_input_ids,
            condition_attention_mask=condition_attention_mask,
            condition_position_ids=condition_position_ids,
        )
        return ClassificationOutputs(outputs=outputs)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_mistral_infos = {
    # mistral
    "mistral-7b-instruct-v0.1": {
        "config": hf_endpoint_url(
            "/filipealmeida/Mistral-7B-Instruct-v0.1-sharded/resolve/main/config.json"
        ),
        "vocab": hf_endpoint_url(
            "/filipealmeida/Mistral-7B-Instruct-v0.1-sharded/resolve/main/tokenizer.model"
        ),
        "weight": [
            hf_endpoint_url(
                f"/filipealmeida/Mistral-7B-Instruct-v0.1-sharded/resolve/main/pytorch_model-{str(i).rjust(5, '0')}-of-00008.bin"
            )
            for i in range(1, 9)
        ],
    },
    "mistral-7b-instruct-v0.3": {
        "config": hf_endpoint_url(
            "/MaziyarPanahi/Mistral-7B-Instruct-v0.3/resolve/main/config.json"
        ),
        "vocab": hf_endpoint_url(
            "/MaziyarPanahi/Mistral-7B-Instruct-v0.3/resolve/main/tokenizer.model"
        ),
        "weight": [
            hf_endpoint_url(
                f"/MaziyarPanahi/Mistral-7B-Instruct-v0.3/resolve/main/model-{str(i).rjust(5, '0')}-of-00003.safetensors"
            )
            for i in range(1, 4)
        ],
    },
}

pretrained_mistral_extensions_infos = {}

import unitorch.cli.models.mistral.modeling
import unitorch.cli.models.mistral.processing
from unitorch.cli.models.mistral.modeling import (
    MistralForClassification,
    MistralForGeneration,
)
from unitorch.cli.models.mistral.processing import MistralProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.dinov2.modeling import DinoV2ForImageClassification
from unitorch.models.dinov2.processing import DinoV2Processor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import json
import random
import logging
import torch
import torch.nn.functional as F
import torch.nn as nn
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers.models.beit import BeitConfig, BeitModel
from unitorch.models import GenericModel


class BeitForImageClassification(GenericModel):
    def __init__(
        self,
        config_path: str,
        num_classes: Optional[int] = 1,
    ):
        """
        Initializes the BeitForImageClassification model.

        Args:
            config_path (str): The path to the configuration file.
            num_classes (Optional[int], optional): The number of classes for classification. Defaults to 1.
        """
        super().__init__()
        config = BeitConfig.from_json_file(config_path)

        self.beit = BeitModel(config, add_pooling_layer=True)
        self.classifier = nn.Linear(config.hidden_size, num_classes)
        self.init_weights()

    def forward(
        self,
        pixel_values: torch.Tensor,
    ):
        """
        Forward pass of the BeitForImageClassification model.

        Args:
            pixel_values (torch.Tensor): The input tensor of pixel values.

        Returns:
            (torch.Tensor):The logits of the model output.
        """
        outputs = self.beit(
            pixel_values=pixel_values,
        )

        pooled_output = outputs[1]
        logits = self.classifier(pooled_output)
        return logits



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_blip_infos = {
    "blip-vqa-base": {
        "config": hf_endpoint_url("/Salesforce/blip-vqa-base/resolve/main/config.json"),
        "vision_config": hf_endpoint_url(
            "/Salesforce/blip-vqa-base/resolve/main/preprocessor_config.json"
        ),
        "vocab": hf_endpoint_url("/Salesforce/blip-vqa-base/resolve/main/vocab.txt"),
        "weight": hf_endpoint_url(
            "/Salesforce/blip-vqa-base/resolve/main/pytorch_model.bin"
        ),
    },
    "blip-image-captioning-base": {
        "config": hf_endpoint_url(
            "/Salesforce/blip-image-captioning-base/resolve/main/config.json"
        ),
        "vision_config": hf_endpoint_url(
            "/Salesforce/blip-image-captioning-base/resolve/main/preprocessor_config.json"
        ),
        "vocab": hf_endpoint_url(
            "/Salesforce/blip-image-captioning-base/resolve/main/vocab.txt"
        ),
        "weight": hf_endpoint_url(
            "/Salesforce/blip-image-captioning-base/resolve/main/pytorch_model.bin"
        ),
    },
    "blip-image-captioning-large": {
        "config": hf_endpoint_url(
            "/Salesforce/blip-image-captioning-large/resolve/main/config.json"
        ),
        "vision_config": hf_endpoint_url(
            "/Salesforce/blip-image-captioning-large/resolve/main/preprocessor_config.json"
        ),
        "vocab": hf_endpoint_url(
            "/Salesforce/blip-image-captioning-large/resolve/main/vocab.txt"
        ),
        "weight": hf_endpoint_url(
            "/Salesforce/blip-image-captioning-large/resolve/main/pytorch_model.bin"
        ),
    },
}

import unitorch.cli.models.blip.modeling
import unitorch.cli.models.blip.processing
from unitorch.cli.models.blip.modeling import (
    BlipForPretrain,
    BlipForClassification,
    BlipForTextClassification,
    BlipForImageClassification,
    BlipForImageCaption,
)
from unitorch.cli.models.blip.processing import BlipProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

# Anime2sketch
# https://github.com/Mukosame/Anime2Sketch

import os
import cv2
import numpy as np
import torch
import torch.nn as nn
import functools
from PIL import Image
from torchvision.transforms import ToTensor, ToPILImage
from unitorch.models import GenericModel
from unitorch.cli import hf_endpoint_url


class UnetSkipConnectionBlock(nn.Module):
    """Defines the Unet submodule with skip connection.
    X -------------------identity----------------------
    |-- downsampling -- |submodule| -- upsampling --|
    """

    def __init__(
        self,
        outer_nc,
        inner_nc,
        input_nc=None,
        submodule=None,
        outermost=False,
        innermost=False,
        norm_layer=nn.BatchNorm2d,
        use_dropout=False,
    ):
        """Construct a Unet submodule with skip connections.
        Parameters:
            outer_nc (int) -- the number of filters in the outer conv layer
            inner_nc (int) -- the number of filters in the inner conv layer
            input_nc (int) -- the number of channels in input images/features
            submodule (UnetSkipConnectionBlock) -- previously defined submodules
            outermost (bool)    -- if this module is the outermost module
            innermost (bool)    -- if this module is the innermost module
            norm_layer          -- normalization layer
            use_dropout (bool)  -- if use dropout layers.
        """
        super(UnetSkipConnectionBlock, self).__init__()
        self.outermost = outermost
        if type(norm_layer) == functools.partial:
            use_bias = norm_layer.func == nn.InstanceNorm2d
        else:
            use_bias = norm_layer == nn.InstanceNorm2d
        if input_nc is None:
            input_nc = outer_nc
        downconv = nn.Conv2d(
            input_nc, inner_nc, kernel_size=4, stride=2, padding=1, bias=use_bias
        )
        downrelu = nn.LeakyReLU(0.2, True)
        downnorm = norm_layer(inner_nc)
        uprelu = nn.ReLU(True)
        upnorm = norm_layer(outer_nc)

        if outermost:
            upconv = nn.ConvTranspose2d(
                inner_nc * 2, outer_nc, kernel_size=4, stride=2, padding=1
            )
            down = [downconv]
            up = [uprelu, upconv, nn.Tanh()]
            model = down + [submodule] + up
        elif innermost:
            upconv = nn.ConvTranspose2d(
                inner_nc, outer_nc, kernel_size=4, stride=2, padding=1, bias=use_bias
            )
            down = [downrelu, downconv]
            up = [uprelu, upconv, upnorm]
            model = down + up
        else:
            upconv = nn.ConvTranspose2d(
                inner_nc * 2,
                outer_nc,
                kernel_size=4,
                stride=2,
                padding=1,
                bias=use_bias,
            )
            down = [downrelu, downconv, downnorm]
            up = [uprelu, upconv, upnorm]

            if use_dropout:
                model = down + [submodule] + up + [nn.Dropout(0.5)]
            else:
                model = down + [submodule] + up

        self.model = nn.Sequential(*model)

    def forward(self, x):
        if self.outermost:
            return self.model(x)
        else:  # add skip connections
            return torch.cat([x, self.model(x)], 1)


class UnetGenerator(GenericModel):
    """Create a Unet-based generator"""

    def __init__(
        self,
        input_nc,
        output_nc,
        num_downs,
        ngf=64,
        norm_layer=nn.BatchNorm2d,
        use_dropout=False,
    ):
        """Construct a Unet generator
        Parameters:
            input_nc (int)  -- the number of channels in input images
            output_nc (int) -- the number of channels in output images
            num_downs (int) -- the number of downsamplings in UNet. For example, # if |num_downs| == 7,
                                image of size 128x128 will become of size 1x1 # at the bottleneck
            ngf (int)       -- the number of filters in the last conv layer
            norm_layer      -- normalization layer
        We construct the U-Net from the innermost layer to the outermost layer.
        It is a recursive process.
        """
        super().__init__()
        # construct unet structure
        unet_block = UnetSkipConnectionBlock(
            ngf * 8,
            ngf * 8,
            input_nc=None,
            submodule=None,
            norm_layer=norm_layer,
            innermost=True,
        )  # add the innermost layer
        for _ in range(num_downs - 5):  # add intermediate layers with ngf * 8 filters
            unet_block = UnetSkipConnectionBlock(
                ngf * 8,
                ngf * 8,
                input_nc=None,
                submodule=unet_block,
                norm_layer=norm_layer,
                use_dropout=use_dropout,
            )
        # gradually reduce the number of filters from ngf * 8 to ngf
        unet_block = UnetSkipConnectionBlock(
            ngf * 4, ngf * 8, input_nc=None, submodule=unet_block, norm_layer=norm_layer
        )
        unet_block = UnetSkipConnectionBlock(
            ngf * 2, ngf * 4, input_nc=None, submodule=unet_block, norm_layer=norm_layer
        )
        unet_block = UnetSkipConnectionBlock(
            ngf, ngf * 2, input_nc=None, submodule=unet_block, norm_layer=norm_layer
        )
        self.model = UnetSkipConnectionBlock(
            output_nc,
            ngf,
            input_nc=input_nc,
            submodule=unet_block,
            outermost=True,
            norm_layer=norm_layer,
        )  # add the outermost layer

    def forward(self, input):
        """Standard forward"""
        return self.model(input)


class LineartAnimeDetector(UnetGenerator):
    def __init__(self):
        norm_layer = functools.partial(
            nn.InstanceNorm2d, affine=False, track_running_stats=False
        )
        super().__init__(3, 1, 8, 64, norm_layer=norm_layer, use_dropout=False)
        self.from_pretrained(
            hf_endpoint_url("/lllyasviel/Annotators/resolve/main/netG.pth"),
            replace_keys={"module.": ""},
        )
        self.eval()
        self.preprocess = ToTensor()
        self.postprocess = ToPILImage()

    @torch.no_grad()
    def __call__(self, input_image):
        input_image = cv2.cvtColor(np.array(input_image), cv2.COLOR_RGB2BGR)
        H, W, C = input_image.shape
        Hn = 256 * int(np.ceil(float(H) / 256.0))
        Wn = 256 * int(np.ceil(float(W) / 256.0))
        img = cv2.resize(input_image, (Wn, Hn), interpolation=cv2.INTER_CUBIC)
        pixel_values = self.preprocess(img).unsqueeze(0) * 2.0 - 1.0

        line = self.forward(pixel_values)[0, 0]
        line = self.postprocess(line.cpu())
        return line


lineart_anime_pipe = None


def lineart_anime(image: Image.Image):
    global lineart_anime_pipe
    if lineart_anime_pipe is None:
        lineart_anime_pipe = LineartAnimeDetector()
        lineart_anime_pipe.to("cpu")
    return lineart_anime_pipe(image)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli.fastapis.controlnet_flux.text2image import (
    ControlNetFluxForText2ImageFastAPIPipeline,
)
from unitorch.cli.fastapis.controlnet_flux.image2image import (
    ControlNetFluxForImage2ImageFastAPIPipeline,
)
from unitorch.cli.fastapis.controlnet_flux.inpainting import (
    ControlNetFluxForImageInpaintingFastAPIPipeline,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_pegasus_infos = {
    "pegasus-cnn_dailymail": {
        "config": hf_endpoint_url(
            "/google/pegasus-cnn_dailymail/resolve/main/config.json"
        ),
        "vocab": hf_endpoint_url(
            "/google/pegasus-cnn_dailymail/resolve/main/spiece.model"
        ),
        "weight": hf_endpoint_url(
            "/google/pegasus-cnn_dailymail/resolve/main/pytorch_model.bin"
        ),
    },
    "pegasus-xsum": {
        "config": hf_endpoint_url("/google/pegasus-xsum/resolve/main/config.json"),
        "vocab": hf_endpoint_url("/google/pegasus-xsum/resolve/main/spiece.model"),
        "weight": hf_endpoint_url(
            "/google/pegasus-xsum/resolve/main/pytorch_model.bin"
        ),
    },
}

import unitorch.cli.models.pegasus.modeling
import unitorch.cli.models.pegasus.processing
from unitorch.cli.models.pegasus.modeling import PegasusForGeneration
from unitorch.cli.models.pegasus.processing import PegasusProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.llava.modeling import (
    LlavaMistralClipForClassification,
    LlavaMistralClipForGeneration,
    LlavaLlamaSiglipForGeneration,
)
from unitorch.models.llava.processing import (
    LlavaMistralClipProcessor,
    LlavaLlamaSiglipProcessor,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli.fastapis.stable_flux.text2image import (
    StableFluxForText2ImageFastAPIPipeline,
)
from unitorch.cli.fastapis.stable_flux.image2image import (
    StableFluxForImage2ImageFastAPIPipeline,
)
from unitorch.cli.fastapis.stable_flux.image_control import (
    StableFluxForImageControlGenerationFastAPIPipeline,
)
from unitorch.cli.fastapis.stable_flux.image_redux import (
    StableFluxForImageReduxGenerationFastAPIPipeline,
)
from unitorch.cli.fastapis.stable_flux.inpainting import (
    StableFluxForImageInpaintingFastAPIPipeline,
)
from unitorch.cli.fastapis.stable_flux.redux_inpainting import (
    StableFluxForReduxInpaintingFastAPIPipeline,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.visualbert.modeling import (
    VisualBertForClassification,
    VisualBertForPretrain,
)
from unitorch.models.visualbert.processing import VisualBertProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from PIL import Image
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.detr import DetrProcessor as _DetrProcessor
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli.models import (
    TensorsInputs,
    ListTensorsInputs,
    DetectionOutputs,
    DetectionTargets,
    SegmentationOutputs,
    SegmentationTargets,
)
from unitorch.cli import WriterOutputs
from unitorch.cli.models.detr import pretrained_detr_infos


class DetrProcessor(_DetrProcessor):
    def __init__(
        self,
        vision_config_path: str,
    ):
        super().__init__(
            vision_config_path=vision_config_path,
        )

    @classmethod
    @add_default_section_for_init("core/process/detr")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/process/detr")
        pretrained_name = config.getoption("pretrained_name", "detr-resnet-50")
        vision_config_path = config.getoption("vision_config_path", None)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(pretrained_detr_infos, pretrained_name, "vision_config"),
        )

        vision_config_path = cached_path(vision_config_path)

        return {
            "vision_config_path": vision_config_path,
        }

    @register_process("core/process/detr/image")
    def _image(
        self,
        image: Union[Image.Image, str],
    ):
        outputs = super().image(
            image=image,
        )
        return ListTensorsInputs(
            images=outputs.image,
        )

    @register_process("core/process/detr/detection")
    def _detection(
        self,
        image: Union[Image.Image, str],
        bboxes: List[List[float]],
        classes: List[int],
        do_eval: Optional[bool] = False,
    ):
        outputs = super().detection(
            image=image,
            bboxes=bboxes,
            classes=classes,
        )
        if do_eval:
            new_h, new_w = outputs.image.size()[1:]
            bboxes = outputs.bboxes
            bboxes[:, 0] = bboxes[:, 0] * new_w
            bboxes[:, 1] = bboxes[:, 1] * new_h
            bboxes[:, 2] = bboxes[:, 2] * new_w
            bboxes[:, 3] = bboxes[:, 3] * new_h
            return ListTensorsInputs(images=outputs.image), DetectionTargets(
                bboxes=bboxes,
                classes=outputs.classes,
            )

        return ListTensorsInputs(
            images=outputs.image,
            bboxes=outputs.bboxes,
            classes=outputs.classes,
        )

    @register_process("core/postprocess/detr/detection")
    def _post_dectection(self, outputs: DetectionOutputs):
        results = outputs.to_pandas()
        results["bboxes"] = [bboxes.tolist() for bboxes in outputs.bboxes]
        results["scores"] = [scores.tolist() for scores in outputs.scores]
        results["classes"] = [classes.tolist() for classes in outputs.classes]
        return WriterOutputs(results)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli.webuis.utils.layouts import (
    create_element,
    create_accordion,
    create_row,
    create_column,
    create_flex_layout,
    create_group,
    create_tab,
    create_tabs,
    create_blocks,
    create_pretrain_layout,
    create_controlnet_layout,
    create_adapter_layout,
    create_lora_layout,
)
